<?php

class WarningException              extends ErrorException {}
class ParseException                extends ErrorException {}
class NoticeException               extends ErrorException {}
class CoreErrorException            extends ErrorException {}
class CoreWarningException          extends ErrorException {}
class CompileErrorException         extends ErrorException {}
class CompileWarningException       extends ErrorException {}
class UserErrorException            extends ErrorException {}
class UserWarningException          extends ErrorException {}
class UserNoticeException           extends ErrorException {}
class StrictException               extends ErrorException {}
class RecoverableErrorException     extends ErrorException {}
class DeprecatedException           extends ErrorException {}
class UserDeprecatedException       extends ErrorException {}

function collect_debug_info($return=false)
{
    if( !isset($_SESSION) )
    {
	$_SESSION = null;
    }
    
    $di = @array(
	'_SERVER' 		=>	$_SERVER,
	'_GET'			=>	$_GET,
	'_POST'			=>	$_POST,
	'_FILES'		=>	$_FILES,
	'_REQUEST'		=>	$_REQUEST,
	'_SESSION'		=>	$_SESSION,
	'_ENV'			=>	$_ENV,
	'_COOKIE'		=>	$_COOKIE,
	'php_errormsg'		=>	$php_errormsg,
	'headers_list()'	=>	headers_list(),
    );
    
    if( $return )
    {
	return $di;
    } else {
	var_export($di);
    }
}

if( GWEFAN_DEBUG )
{
    //Show explicit error messages..
    ini_set('display_errors', 1);
    error_reporting(E_ALL | E_STRICT);

    set_error_handler(
	function ($err_severity, $err_msg, $err_file, $err_line, array $err_context)
	{
	    //Error was suppressed with the @-operator
	    if( 0 === error_reporting() )
	    {
		return false;
	    }
	    
	    switch($err_severity)
	    {
		case E_ERROR:               throw new ErrorException            ($err_msg, 0, $err_severity, $err_file, $err_line);
		case E_WARNING:             throw new WarningException          ($err_msg, 0, $err_severity, $err_file, $err_line);
		case E_PARSE:               throw new ParseException            ($err_msg, 0, $err_severity, $err_file, $err_line);
		case E_NOTICE:              throw new NoticeException           ($err_msg, 0, $err_severity, $err_file, $err_line);
		case E_CORE_ERROR:          throw new CoreErrorException        ($err_msg, 0, $err_severity, $err_file, $err_line);
		case E_CORE_WARNING:        throw new CoreWarningException      ($err_msg, 0, $err_severity, $err_file, $err_line);
		case E_COMPILE_ERROR:       throw new CompileErrorException     ($err_msg, 0, $err_severity, $err_file, $err_line);
		case E_COMPILE_WARNING:     throw new CoreWarningException      ($err_msg, 0, $err_severity, $err_file, $err_line);
		case E_USER_ERROR:          throw new UserErrorException        ($err_msg, 0, $err_severity, $err_file, $err_line);
		case E_USER_WARNING:        throw new UserWarningException      ($err_msg, 0, $err_severity, $err_file, $err_line);
		case E_USER_NOTICE:         throw new UserNoticeException       ($err_msg, 0, $err_severity, $err_file, $err_line);
		case E_STRICT:              throw new StrictException           ($err_msg, 0, $err_severity, $err_file, $err_line);
		case E_RECOVERABLE_ERROR:   throw new RecoverableErrorException ($err_msg, 0, $err_severity, $err_file, $err_line);
		case E_DEPRECATED:          throw new DeprecatedException       ($err_msg, 0, $err_severity, $err_file, $err_line);
		case E_USER_DEPRECATED:     throw new UserDeprecatedException   ($err_msg, 0, $err_severity, $err_file, $err_line);
	    }
	}
    );
} else {
    //Hide error messages and send email instead
    ini_set('display_errors', 0);
    error_reporting(E_ALL & ~E_NOTICE & ~E_STRICT & ~E_DEPRECATED);
    
    set_error_handler(
	function ($err_severity, $err_msg, $err_file, $err_line, array $err_context)
	{
	    $error = array(
		'severity'	=>	$err_severity,
		'msg'		=>	$err_msg,
		'file'		=>	$err_file,
		'line'		=>	$err_line,
		'context'	=>	$err_context,
	    );
	    $error = var_export($error, true);
	    $error .= PHP_EOL;
	    $error .= var_export(collect_debug_info(true), true);
	    $debug_email = GWEFAN_DEBUG_EMAIL_ADDRESS;
	    if( !empty($debug_email) )
	    {
		 mail(GWEFAN_DEBUG_EMAIL_ADDRESS, GWEFAN_DEBUG_EMAIL_PREFIX.$err_msg, $error);
	    }
	    
	    print '<pre><b style="color:#F00">(ERROR)</b> Sorry, an error has occured on this page. The site owner has been notified.</pre>';
	    
	    exit(1);
	}
    );
    
    set_exception_handler(
	function ($exception)
	{
	    $error = var_export($exception, true);
	    $error .= PHP_EOL;
	    $error .= var_export(collect_debug_info(true), true);
	    $debug_email = GWEFAN_DEBUG_EMAIL_ADDRESS;
	    if( !empty($debug_email) )
	    {
		 mail(GWEFAN_DEBUG_EMAIL_ADDRESS, GWEFAN_DEBUG_EMAIL_PREFIX.get_class($exception), $error);
	    }
	    
	    print '<pre><b style="color:#F00">(ERROR)</b> Sorry, an error has occured on this page. The site owner has been notified.</pre>';
	    
	    exit(1);
	}
    );    
}
?>